//Datei: DS1821.C
// Hauptprogramm
//
// Serielles Thermometer
// mit PIC12F675 und DS1821
//
// Copyright 2003
// 23.01.2003
// Compiler CC5x
// hk@holger-klabunde.de
// http://www.holger-klabunde.de

#include "prozess.h"      /* Der verwendete Prozessor */

#include "ser1821.h"
#include "protos.h"
#include <math16.h>

//Statt einer Linkerdatei mssen alle Module #included werden
#include "serial.c"
#include "delay.c"
#include "binbcd16.c"
#include "ds1821io.c"

//Globale Flags und Rckgabevariablen

//long ist 16Bit !
int sign;       //Vorzeichen und 7Bit Temperaturwert
long temp;     //Temperatur
long countrem; //9 Bit Register 1
long countdeg; //9 Bit Register 2
long tempbuf;

unsigned char by,i,mask;

/* Hauptprogramm */
void main()
{
 
// OSCCAL=W; //Als erstes den Kalibrierwert fr den int. RC-Oszi sichern

//##### Achtung #### Ports sind sonst keine digitalen IO-Pins !!
 CMCON=0x07; //alles digitale IO-Pins
 ANSEL=0x00; //alles digitale IO-Pins
 WPU=0;      //Pullups off
 

 CLRWDT;
 //Takt des PIC 4MHz, ein Zyklus = 1us
 OPTION=0b11000001; //Pullups off,Prescaler auf TMR0, 1:4


 Delay1ms(250); //Power up abwarten

 TRISIO=0b111010;  //0=Ausgang, 1=Eingang
 GPIO=0b111000;  //Port einmal schreiben, DS1821 einschalten

 //Serielle Schnittstelle einstellen
 TX=0;
 Delay1ms(250);
 Delay1ms(250);

//"DS" zurcksenden
// Transmit('D'); Transmit('S'); Transmit(0x0d); Transmit(0x0a);

 MasterReset();
 SendByte(WRITESTATUS);
 SendByte(0x41); //One Shot Operation einstellen
 Delay1ms(250);

 while(1) // Anfang Programm Endlosschleife
  {
   MasterReset();      //Messung starten
   SendByte(STARTCONV);

 Delay1ms(250);
 Delay1ms(250);
 Delay1ms(250);
 Delay1ms(250);
//Eine Messung pro Sekunde

  //Hier Status Polling fr Ende der Messung
   PollStatus();
   
   MasterReset();
   SendByte(READTEMP);
   ReadByte(8);       //Temperatur lesen
   sign=(int)(tempbuf&0xFF); //Nur die unteren 8 Bits nehmen !
   
   //Die beiden 9 Bit Register lesen
   MasterReset();
   SendByte(READCOUNT);
   ReadByte(9);       //Remain lesen
   countrem=tempbuf;

   MasterReset();
   SendByte(LOADREM); //Zweites Register laden

   MasterReset();
   SendByte(READCOUNT);
   ReadByte(9);       //Degree lesen
   countdeg=tempbuf;
   
   //signed integer aus Zweierkomplement machen wenn negativ
   if(sign<0)
    {
     sign=sign+255;     //Einerkomplement
     sign++;            //Zweierkomplement
    }

   temp=(long)(sign); //int zu long

   //Temperatur * 100  => -5500 bis 12500
   //Schiebt den Wert um zwei Dezimalstellen nach links
   //So spare ich mir unten eine Subtraktion und eine Addition
   //im Floating Point Format
   tempbuf=temp;
   for(i=0; i<99; i++) temp+=tempbuf;


//Wenn countdeg=0 mte durch 0 geteilt werden. Abfangen
   if(countdeg>(long)0) //Schade jetzt mu ich rechnen
    {
     //countdeg-countrem in integer rechnen Ergebnis in countrem
     countrem=countdeg-countrem;
     //Das Ergebnis ist immer positiv

     //Ich mchte zwei Stellen hinter dem Komma haben
     //Das Ergebnis der Division wre aber kleiner 1
     //Um wieder FloatingPoint Operation zu vermeiden
     //nehme ich den Zhler * 100
     tempbuf=countrem;
     for(i=0; i<99; i++) countrem+=tempbuf;

     countrem=countrem/countdeg; //Und teilen
    } 
   else //countdeg war 0, dann 100 * 1/2 LSB(=0.5) addieren
    { countrem=(long)50; } //1/2 LSB
   
   temp+=countrem; //Genaue Temperatur ausrechnen
   //LSB = 0.5 * 100 = 50
   //Temperatur - LSB ausrechnen
   temp-=(long)50;

   //Temperatur fnfstellig anzeigen
 if(temp<0) //Falls negativ, Absolutwert anzeigen
  {
   temp=-temp;
   Transmit('-'); //Vorzeichen - auf Display
  }
 else
  {
   Transmit('+'); //Vorzeichen + auf Display
  } 

 tmp1=(unsigned char)(temp>>8); //Highbyte
 tmp=(unsigned char)(temp&0xFF); //Lowbyte

 BinToBCD16(); //unsigned int zu BCD

 Transmit(R0+0x30); //Zehntausender
 ShowBCD(R1); //Tausender Hunderter
 Transmit('.'); //"." anzeigen
 ShowBCD(R2); //Zehner Einer
 Transmit(0x0d); //CR
 Transmit(0x0a); //LF

 for(i=0; i<20; i++);  Delay1ms(250); //Warte 5s bis zur nchsten Messung
 //Der DS1821 erwrmt sich selbst wenn er zu lange arbeitet !

  } // Ende Programm Endlosschleife

}

